@@ -40,6 +40,7 @@
 #include "panel-background.h"
 #include "panel-lockdown.h"
 #include "panel-stock-icons.h"
+#include "xstuff.h"
 
 #define PANEL_APPLET_FRAME_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_APPLET_FRAME, PanelAppletFramePrivate))
 
@@ -572,11 +573,11 @@ panel_applet_frame_paint (GtkWidget    *widget,
 		switch (frame->priv->orientation) {
 		case PANEL_ORIENTATION_TOP:
 		case PANEL_ORIENTATION_BOTTOM:
-			orientation = GTK_ORIENTATION_HORIZONTAL;
+			orientation = GTK_ORIENTATION_VERTICAL;
 			break;
 		case PANEL_ORIENTATION_LEFT:
 		case PANEL_ORIENTATION_RIGHT:
-			orientation = GTK_ORIENTATION_VERTICAL;
+			orientation = GTK_ORIENTATION_HORIZONTAL;
 			break;
 		default:
 			g_assert_not_reached ();
@@ -945,6 +946,9 @@ panel_applet_frame_cnx_broken (ORBitConnection  *cnx,
 
 	screen = gtk_widget_get_screen (GTK_WIDGET (frame));
 
+	if (xstuff_is_display_dead ())
+		return;
+
 	if (frame->priv->iid)
 		applet_name = panel_applet_frame_get_name (frame->priv->iid);
 
@@ -1285,7 +1289,7 @@ panel_applet_frame_activated (Bonobo_Unknown     object,
 	control_frame = bonobo_widget_get_control_frame (BONOBO_WIDGET (widget));
 	if (control_frame == NULL) {
 		g_warning (G_STRLOC ": failed to load applet %s "
-			   "(can't get control frame)", frame->priv->iid);
+			   "(cannot get control frame)", frame->priv->iid);
 		panel_applet_frame_loading_failed (frame, frame_act->id);
 		gtk_object_sink (GTK_OBJECT (widget));
 		g_free (frame_act->id);
@@ -1300,7 +1304,7 @@ panel_applet_frame_activated (Bonobo_Unknown     object,
 		error = bonobo_exception_get_text (&corba_ev);
 		CORBA_exception_free (&corba_ev);
 		g_warning (G_STRLOC ": failed to load applet %s "
-			   "(can't get property bag):\n%s",
+			   "(cannot get property bag):\n%s",
 			   frame->priv->iid, error);
 		panel_applet_frame_loading_failed (frame, frame_act->id);
 		gtk_object_sink (GTK_OBJECT (widget));
@@ -1323,7 +1327,7 @@ panel_applet_frame_activated (Bonobo_Unknown     object,
 		error = bonobo_exception_get_text (&corba_ev);
 		CORBA_exception_free (&corba_ev);
 		g_warning (G_STRLOC ": failed to load applet %s "
-			   "(can't get popup component):\n%s",
+			   "(cannot get popup component):\n%s",
 			   frame->priv->iid, error);
 		panel_applet_frame_loading_failed (frame, frame_act->id);
 		gtk_object_sink (GTK_OBJECT (widget));
@@ -1348,7 +1352,7 @@ panel_applet_frame_activated (Bonobo_Unknown     object,
 	if (!control) {
 		CORBA_exception_free (&corba_ev);
 		g_warning (G_STRLOC ": failed to load applet %s "
-			   "(can't get control)", frame->priv->iid);
+			   "(cannot get control)", frame->priv->iid);
 		panel_applet_frame_loading_failed (frame, frame_act->id);
 		gtk_object_sink (GTK_OBJECT (widget));
 		g_free (frame_act->id);
@@ -1360,7 +1364,7 @@ panel_applet_frame_activated (Bonobo_Unknown     object,
 	if (frame->priv->applet_shell == CORBA_OBJECT_NIL) {
 		CORBA_exception_free (&corba_ev);
 		g_warning (G_STRLOC ": failed to load applet %s "
-			   "(can't get applet shell)", frame->priv->iid);
+			   "(cannot get applet shell)", frame->priv->iid);
 		panel_applet_frame_loading_failed (frame, frame_act->id);
 		gtk_object_sink (GTK_OBJECT (widget));
 		g_free (frame_act->id);
