@@ -684,8 +684,10 @@ panel_menu_button_load (const char  *menu_path,
 
 	panel_applet_add_callback (info, "help", GTK_STOCK_HELP, _("_Help"), NULL);
 
-        if (panel_is_program_in_path ("gmenu-simple-editor"))
-		panel_applet_add_callback (info, "edit", NULL, _("_Edit Menus"), NULL);
+        if (panel_is_program_in_path ("alacarte") ||
+	    panel_is_program_in_path ("gmenu-simple-editor"))
+		panel_applet_add_callback (info, "edit", NULL,
+					   _("_Edit Menus"), NULL);
 
 	panel_widget_set_applet_expandable (panel, GTK_WIDGET (button), FALSE, TRUE);
 	panel_widget_set_applet_size_constrained (panel, GTK_WIDGET (button), TRUE);
@@ -997,17 +999,27 @@ panel_menu_button_invoke_menu (PanelMenuButton *button,
 	} else if (!strcmp (callback_name, "edit")) {
                 GError *error = NULL;
 
-		panel_launch_desktop_file ("gmenu-simple-editor.desktop",
-					   "gmenu-simple-editor",
+		panel_launch_desktop_file ("alacarte.desktop",
+					   "alacarte-editor",
 					   screen,
 					   &error);
+		if (error) {
+			g_error_free (error);
+			panel_launch_desktop_file ("gmenu-simple-editor.desktop",
+						   "gmenu-simple-editor",
+						   screen,
+						   &error);
+		}
+
                 if (error) {
-                        panel_error_dialog (screen,
-                                            "cannot_exec_gmenu-simple-editor", TRUE,
-                                            _("Cannot execute '%s'"),
-                                            "%s",
-                                            "gmenu-simple-editor",
-                                            error->message);
+			char *primary;
+			primary = g_strdup_printf (_("Could not execute '%s'"),
+						   "gmenu-simple-editor");
+                        panel_error_dialog (NULL, screen,
+                                            "cannot_exec_gmenu-simple-editor",
+					    TRUE,
+					    primary, error->message);
+			g_free (primary);
                         g_error_free (error);
                 }
 	}
@@ -1034,9 +1046,14 @@ panel_menu_button_set_dnd_enabled (PanelMenuButton *button,
 		gtk_drag_source_set (GTK_WIDGET (button), GDK_BUTTON1_MASK,
 				     dnd_targets, 1,
 				     GDK_ACTION_COPY | GDK_ACTION_MOVE);
+
 		icon = panel_menu_button_get_icon (button);
-		gtk_drag_source_set_icon_name (GTK_WIDGET (button), icon);
-		g_free (icon);
+		if (icon != NULL) {
+			gtk_drag_source_set_icon_name (GTK_WIDGET (button),
+						       icon);
+			g_free (icon);
+		}
+
 		GTK_WIDGET_SET_FLAGS (button, GTK_NO_WINDOW);
 	} else
 		gtk_drag_source_unset (GTK_WIDGET (button));
